package checkers.nonnull;

import javax.lang.model.element.AnnotationMirror;

import checkers.basetype.BaseTypeChecker;
import checkers.basetype.BaseTypeVisitor;
import checkers.source.Result;
import checkers.types.AnnotatedTypeMirror;

import com.sun.source.tree.CompilationUnitTree;
import com.sun.source.tree.MemberSelectTree;

/**
 * A type-checking visitor for the {@link checkers.quals.NonNull} type
 * qualifier that uses the {@link BaseTypeVisitor} implementation. This visitor
 * reports errors or warnings for violations for the following cases:
 *
 * <ul>
 * <li>if the receiver of a member dereference is not NonNull (error
 * "deref.invalid")</li>
 * </ul>
 */
public class NonNullVisitor extends BaseTypeVisitor<Void, Void> {

    private final AnnotationMirror NONNULL;

    public NonNullVisitor(BaseTypeChecker checker, CompilationUnitTree root) {
        super(checker, root);
        NONNULL = this.annoFactory.fromName("checkers.quals.NonNull");
    }

    @Override
    public Void visitMemberSelect(MemberSelectTree node, Void p) {
        
        AnnotatedTypeMirror type = factory.getAnnotatedType(node.getExpression());
        
        // TODO check element kind
        //Element elt = type.getElement();
                  
        if (!type.hasAnnotation(NONNULL))
            checker.report(Result.failure("deref.invalid"), node);
        
        return super.visitMemberSelect(node, p);
    }
    
}
