package tests;

import org.junit.*;
import static org.junit.Assert.*;

import javax.tools.*;
import java.util.*;

import com.sun.source.tree.*;

/**
 * JUnit tests for the Interned annotation checker.
 */
public class InternedTest extends CheckerTest {

    public InternedTest() {
        checkerName = "checkers.interned.InternedChecker";
        checkerDir = "interned";
    }

    /** Tests arrays. */
    @Test public void testArrays() {
        test(true);
    }

    /** Tests the basic semantics of Interned. */
    @Test public void testComparison() {
        test(false);
    }

    /** Tests Interned checks in complex conditional expressions. */
    @Test public void testComplexComparison() {
        test(false);
    }

    /** Tests that Interned works properly with object creation expressions. */
    @Test public void testCreation() {
        test(false);
    }

    /** Tests that Interned works properly with enumerated types. */
    @Test public void testEnumerations() {
        test(true);
    }

    /** Tests that Interned works properly with expressions. */
    @Test public void testExpressions() {
        test(true);
    }

    /** Tests that Interned works properly with generic types. */
    @Test public void testGenerics() {
        test(false);
    }

    /** Tests a standard usage scenario. */
    @Test public void testInternMethod() {
        test(true);
    }

    /** Tests a standard usage scenario. */
    @Test public void testInternedClass() {
        test(true);
    }

    /** Tests that primitive types are implicitly treated as Interned. */
    @Test public void testMissingThisAnnotated() {
        test(false);
    }

    /** Tests that primitive types are implicitly treated as Interned. */
    @Test public void testPrimitives() {
        test(true);
    }

    /** Tests another standard usage scenario. */
    @Test public void testStaticInternMethod() {
        test(true);
    }

    /** Tests that the return type of String.intern is implicitly Interned. */
    @Test public void testStringIntern() {
        test(false);
    }

//    /** Tests that SuppressWarnings works on variable declarations. */
//    @Test public void testSubclass() {
//        TestRun run = getTest("Subclass.java");
//        assertSuccess(run);
//        List<Diagnostic<? extends JavaFileObject>> list = run.getDiagnostics();
//        assertDiagnostics(list, "Subclass.out");
//    }

    /** Tests that SuppressWarnings works on variable declarations. */
    @Test public void testSuppressWarningsVar() {
        test(true);
    }

    /** Tests that SuppressWarnings works on class declarations. */
    @Test public void testSuppressWarningsClass() {
        test(true);
    }

    @Test public void testNestedGenerics() {
        test(true);
    }
}
